## Optic CI

Installation

```bash
npm install -D @useoptic/optic-ci
```

```bash
yarn add -D @useoptic/optic-ci
```

`optic-ci` reads config from the folder it's run from - this is used to configure the changelog, and rules. By signing up for the beta you can get an OPTIC_TOKEN which will allow you to visualize changes between APIs and the rules that your team has defined.

```javascript
// optic.config.js
module.exports = {
  token: process.env.OPTIC_TOKEN,
  gitProvider: {
    token: process.env.GITHUB_TOKEN,
  },
};
```

## Running a comparison

You can run a comparison between two OpenAPI specs. `from` and `to` take in a path to a file (`specs/openapi.yml`), or a git ref and path to file (`master:specs/openapi.yml`).

`optic-ci compare --from <fromref> --to <toref> --upload-results`


## Bulk comparisons

If you have multiple openapi specs, you can specify glob patterns and ignore patterns (we use [picomatch](https://github.com/micromatch/picomatch) under the hood). `glob` and `ignore` take a comma separated list of globs (e.g. `--glob '**/*.yml,**/*.json'` matches any `.yml` or `.json` file).

`optic-ci bulk-compare --glob <glob> --ignore <ignore> --upload-results`


## Standard rulesets

Optic comes bundled with standard rulesets ([details here](../standard-rulesets/README.md)). You can install this by running:

```
npm install -D @useoptic/standard-rulesets
```

```
yarn add -D @useoptic/standard-rulesets
```

### Catch breaking changes between versions

Turn on breaking changes detection between every API change.

```javascript
const { BreakingChangesRuleset } = require('@useoptic/standard-rulesets');

module.exports = {
  // optic.config.js
  ...
  rules: [
    new BreakingChangesRuleset()
  ],
}
```

### Apply your team's naming checks

Turn on breaking changes detection between every API change.

```javascript
const { NamingChangesRuleset } = require('@useoptic/standard-rulesets');

module.exports = {
  // optic.config.js
  ...
  rules: [
    new NamingChangesRuleset({
      required_on: 'always', // also available: 'added' | 'addedOrChanged'
      options: { // valid formats are: 'camelCase' | 'Capital-Param-Case' | 'param-case' | 'PascalCase' | 'snake_case'
        properties: 'camelCase',
        queryParameters: 'camelCase',
        requestHeaders: 'camelCase',
        responseHeaders: 'camelCase',
      }
    })
  ],
}
```

### Exemptions

Any rule can be locally skipped by applying a `x-optic-exemptions` tag to the OpenAPI specification node.

For instance if `optic-ci` emits such an error:

```
   FAIL  POST /pet/{petId}/uploadImage
    Rule: prevent new required cookie parameters
    x added cookie parameter: DebugCookie in operation: POST /pet/{petId}/uploadImage must not add required cookie parameter
      cannot add a required cookie parameter to an existing operation
```

But for some reason you wish to bypass this rule for this node, you can add an `x-optic-exemptions` array property to the node containing the name of the rule you wish to bypass.

```json
{
  "paths": {
    "/pet/{petId}/uploadImage": {
      "post": {
        [...]
        "x-optic-exemptions": ["prevent new required cookie parameters"] <- Here
      }
    }
  }
}
```

The rule won't be applied to this particular node, and `optic-ci` will report how many rules were exempted in the logs:

```
44 changes detected
94 checks passed
3 checks failed
2 checks exempted <- Here
```

### Join the beta!

We're exploring how to make writing OpenAPI, writing optic rules and standardizing APIs easy for everyone - join our beta and let us help you:

- Drive adoption of OpenAPI in your team
- Write custom rules and checks specific to your team's standards

You can join by signing up on our [website](https://www.useoptic.com/).
